from django.conf import settings


from djangotools.common.errors import UnauthorizedException
import datetime


import sys
from collections import defaultdict
from pathlib import Path


import json

from djangotools.common.errors import UnauthorizedException

from django.conf import settings
from djangotools.common.module_loader import iter_module_content
from djangotools.common import get_url, redirect

allow_all=False


def get_settings(request):
    if getattr(settings, "PRODUCTION", True):
        return {
            "url": ""
        }
    else:
        return {
                "url" : f"http://localhost:{request.META['SERVER_PORT']}",
            }


base_url = settings.BASE_URL
if base_url and not base_url.startswith("/"): base_url = "/"+base_url
#if base_url.endswith("/"): base_url = base_url[:-1]
class ContextData:
    page = None
    need_auth = True
    redirect = True
    action_params = True
    preferences_class = None
    default_action = None
    context_to_string = True
    redirect_login = False

    def is_mobile(self, request):
        ua = request.headers.get("user-agent", "").lower()
        return any(x in ua for x in [
            "mobile", "android", "iphone"
        ])

    def get_context(self, request,  **kwargs):

        from djangotools.models.preference import UserPreferenceManager
        self.preferences_class = self.preferences_class or UserPreferenceManager
        request.user.pref = self.preferences_class(request.user)

        if ( not allow_all and self.need_auth) and not request.user.is_authenticated:
            if self.redirect_login:
                return redirect(settings.LOGIN_URL)
            else:
                raise UnauthorizedException("Cette ressource nécessite une authentification")

        body = request.POST or (json.loads(request.body) if  request.body else None)
        if "post_action" in kwargs:
            body = kwargs["post_action"]

        post_data = body or self.default_action
        if post_data and "action" in post_data:
            action_params = PostParameters(request, post_data, **kwargs) if (body or self.default_action) and self.action_params else None
        else:
            action_params = None
        include_data = kwargs.get("include_data", True)

        if include_data:
            data = self.get_context_data(request, action_params, **kwargs)
        else:
            data = {}
        if "data" in kwargs: data.update(kwargs.pop("data"))

        return {
            "context" : {
                "page" : self.page,
                "data" : data,
                "base_url" : base_url,
                "user" : {
                    "authenticated" : request.user.is_authenticated,
                    "name" : request.user.username,
                    "id" : request.user.id,
                    "preferences" : request.user.pref.dict,
                    "profile" : request.user.pref.profile,
                    "device": {
                        "mobile" : self.is_mobile(request)
                    }
                },
                "server" : get_settings(request)
,                **kwargs
            }
        }

    def get_context_data(self, request, action_params,  **kwargs):
        return {}



CONTEXT="context"
ACTION_PARAMS="action"
def load_module(module):

    # module_path = "mypackage.%s" % module
    module_path = module

    if module_path in sys.modules:
        return sys.modules[module_path]

    return __import__(module_path, fromlist=[module])



def load_modules():
    global modules
    for name, curr in iter_module_content(settings.CONTEXT_MODULES, classe=(ContextData, ActionParams)):
        if issubclass(curr, ContextData):
            modules[CONTEXT][curr.page] = curr
        else:
            if isinstance(curr.action, str):
                modules[ACTION_PARAMS][curr.action] = curr
            elif isinstance(curr.action, (list, tuple)):
                for act in curr.action:
                    modules[ACTION_PARAMS][act] = curr

def get_context_class(name):
    if name in modules[CONTEXT]:
        return modules[CONTEXT][name]

def get_action_params_class(name):
    if name in modules[ACTION_PARAMS]:
        return modules[ACTION_PARAMS][name]


class ActionParams:

    action = None

    def __init__(self, request, action, data, **kwargs):
        self._action = action
        self._data = data
        self._request=request

    @property
    def user(self):
        return self._request.user


    def get(self, name, default=None):
        return self._data.get(name, default)


class PostParameters:

    def __init__(self, request, content_data, **kwargs):
        self._data = content_data
        self.action = content_data["action"]
        self.request = request
        action_class = get_action_params_class(self.action)
        if not action_class:
            raise ValueError(f"Action '{self.action}' inconnue....")
        self.params = action_class(request, self.action, content_data["params"], **kwargs)
        self.options = content_data.get("options", {})



modules = defaultdict(dict)

